# coding: utf-8

# flake8: noqa
"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from lusid_workflow.models.access_controlled_action import AccessControlledAction
from lusid_workflow.models.access_controlled_resource import AccessControlledResource
from lusid_workflow.models.action_definition import ActionDefinition
from lusid_workflow.models.action_definition_response import ActionDefinitionResponse
from lusid_workflow.models.action_details import ActionDetails
from lusid_workflow.models.action_details_response import ActionDetailsResponse
from lusid_workflow.models.action_id import ActionId
from lusid_workflow.models.create_child_task_configuration import CreateChildTaskConfiguration
from lusid_workflow.models.create_child_tasks_action import CreateChildTasksAction
from lusid_workflow.models.create_child_tasks_action_response import CreateChildTasksActionResponse
from lusid_workflow.models.create_event_handler_request import CreateEventHandlerRequest
from lusid_workflow.models.create_new_task_activity import CreateNewTaskActivity
from lusid_workflow.models.create_new_task_activity_response import CreateNewTaskActivityResponse
from lusid_workflow.models.create_task_definition_request import CreateTaskDefinitionRequest
from lusid_workflow.models.create_task_request import CreateTaskRequest
from lusid_workflow.models.create_worker_request import CreateWorkerRequest
from lusid_workflow.models.deleted_entity_response import DeletedEntityResponse
from lusid_workflow.models.event_handler import EventHandler
from lusid_workflow.models.event_handler_mapping import EventHandlerMapping
from lusid_workflow.models.event_matching_pattern import EventMatchingPattern
from lusid_workflow.models.fail import Fail
from lusid_workflow.models.fail_response import FailResponse
from lusid_workflow.models.field_mapping import FieldMapping
from lusid_workflow.models.get_worker_result_response import GetWorkerResultResponse
from lusid_workflow.models.health_check import HealthCheck
from lusid_workflow.models.health_check_response import HealthCheckResponse
from lusid_workflow.models.id_selector_definition import IdSelectorDefinition
from lusid_workflow.models.identifier_part_schema import IdentifierPartSchema
from lusid_workflow.models.initial_state import InitialState
from lusid_workflow.models.link import Link
from lusid_workflow.models.luminesce_view import LuminesceView
from lusid_workflow.models.luminesce_view_response import LuminesceViewResponse
from lusid_workflow.models.lusid_problem_details import LusidProblemDetails
from lusid_workflow.models.lusid_validation_problem_details import LusidValidationProblemDetails
from lusid_workflow.models.paged_resource_list_of_event_handler import PagedResourceListOfEventHandler
from lusid_workflow.models.paged_resource_list_of_task import PagedResourceListOfTask
from lusid_workflow.models.paged_resource_list_of_task_definition import PagedResourceListOfTaskDefinition
from lusid_workflow.models.paged_resource_list_of_worker import PagedResourceListOfWorker
from lusid_workflow.models.parameter import Parameter
from lusid_workflow.models.parameter_value import ParameterValue
from lusid_workflow.models.read_only_states import ReadOnlyStates
from lusid_workflow.models.resource_id import ResourceId
from lusid_workflow.models.resource_list_of_access_controlled_resource import ResourceListOfAccessControlledResource
from lusid_workflow.models.resource_list_of_task import ResourceListOfTask
from lusid_workflow.models.result_field import ResultField
from lusid_workflow.models.result_matching_pattern import ResultMatchingPattern
from lusid_workflow.models.resultant_child_task_configuration import ResultantChildTaskConfiguration
from lusid_workflow.models.run_worker_action import RunWorkerAction
from lusid_workflow.models.run_worker_action_response import RunWorkerActionResponse
from lusid_workflow.models.run_worker_request import RunWorkerRequest
from lusid_workflow.models.run_worker_response import RunWorkerResponse
from lusid_workflow.models.scheduler_job import SchedulerJob
from lusid_workflow.models.scheduler_job_response import SchedulerJobResponse
from lusid_workflow.models.sleep import Sleep
from lusid_workflow.models.sleep_response import SleepResponse
from lusid_workflow.models.stack import Stack
from lusid_workflow.models.task import Task
from lusid_workflow.models.task_activity import TaskActivity
from lusid_workflow.models.task_activity_response import TaskActivityResponse
from lusid_workflow.models.task_definition import TaskDefinition
from lusid_workflow.models.task_definition_version import TaskDefinitionVersion
from lusid_workflow.models.task_field_definition import TaskFieldDefinition
from lusid_workflow.models.task_instance_field import TaskInstanceField
from lusid_workflow.models.task_state_definition import TaskStateDefinition
from lusid_workflow.models.task_summary import TaskSummary
from lusid_workflow.models.task_transition_definition import TaskTransitionDefinition
from lusid_workflow.models.transition_trigger_definition import TransitionTriggerDefinition
from lusid_workflow.models.trigger_parent_task_action import TriggerParentTaskAction
from lusid_workflow.models.trigger_parent_task_action_response import TriggerParentTaskActionResponse
from lusid_workflow.models.trigger_schema import TriggerSchema
from lusid_workflow.models.update_event_handler_request import UpdateEventHandlerRequest
from lusid_workflow.models.update_matching_tasks_activity import UpdateMatchingTasksActivity
from lusid_workflow.models.update_matching_tasks_activity_response import UpdateMatchingTasksActivityResponse
from lusid_workflow.models.update_task_definition_request import UpdateTaskDefinitionRequest
from lusid_workflow.models.update_task_request import UpdateTaskRequest
from lusid_workflow.models.update_worker_request import UpdateWorkerRequest
from lusid_workflow.models.version_info import VersionInfo
from lusid_workflow.models.worker import Worker
from lusid_workflow.models.worker_configuration import WorkerConfiguration
from lusid_workflow.models.worker_configuration_response import WorkerConfigurationResponse
from lusid_workflow.models.worker_status_triggers import WorkerStatusTriggers


__all__ = [
    "AccessControlledAction",
    "AccessControlledResource",
    "ActionDefinition",
    "ActionDefinitionResponse",
    "ActionDetails",
    "ActionDetailsResponse",
    "ActionId",
    "CreateChildTaskConfiguration",
    "CreateChildTasksAction",
    "CreateChildTasksActionResponse",
    "CreateEventHandlerRequest",
    "CreateNewTaskActivity",
    "CreateNewTaskActivityResponse",
    "CreateTaskDefinitionRequest",
    "CreateTaskRequest",
    "CreateWorkerRequest",
    "DeletedEntityResponse",
    "EventHandler",
    "EventHandlerMapping",
    "EventMatchingPattern",
    "Fail",
    "FailResponse",
    "FieldMapping",
    "GetWorkerResultResponse",
    "HealthCheck",
    "HealthCheckResponse",
    "IdSelectorDefinition",
    "IdentifierPartSchema",
    "InitialState",
    "Link",
    "LuminesceView",
    "LuminesceViewResponse",
    "LusidProblemDetails",
    "LusidValidationProblemDetails",
    "PagedResourceListOfEventHandler",
    "PagedResourceListOfTask",
    "PagedResourceListOfTaskDefinition",
    "PagedResourceListOfWorker",
    "Parameter",
    "ParameterValue",
    "ReadOnlyStates",
    "ResourceId",
    "ResourceListOfAccessControlledResource",
    "ResourceListOfTask",
    "ResultField",
    "ResultMatchingPattern",
    "ResultantChildTaskConfiguration",
    "RunWorkerAction",
    "RunWorkerActionResponse",
    "RunWorkerRequest",
    "RunWorkerResponse",
    "SchedulerJob",
    "SchedulerJobResponse",
    "Sleep",
    "SleepResponse",
    "Stack",
    "Task",
    "TaskActivity",
    "TaskActivityResponse",
    "TaskDefinition",
    "TaskDefinitionVersion",
    "TaskFieldDefinition",
    "TaskInstanceField",
    "TaskStateDefinition",
    "TaskSummary",
    "TaskTransitionDefinition",
    "TransitionTriggerDefinition",
    "TriggerParentTaskAction",
    "TriggerParentTaskActionResponse",
    "TriggerSchema",
    "UpdateEventHandlerRequest",
    "UpdateMatchingTasksActivity",
    "UpdateMatchingTasksActivityResponse",
    "UpdateTaskDefinitionRequest",
    "UpdateTaskRequest",
    "UpdateWorkerRequest",
    "VersionInfo",
    "Worker",
    "WorkerConfiguration",
    "WorkerConfigurationResponse",
    "WorkerStatusTriggers"
]



# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic.v1 import BaseModel, Field, conlist, constr, validator
from lusid_workflow.models.event_handler_mapping import EventHandlerMapping
from lusid_workflow.models.field_mapping import FieldMapping

class CreateNewTaskActivity(BaseModel):
    """
    Define a Task Activity that creates a new task  # noqa: E501
    """
    initial_trigger: constr(strict=True, min_length=1) = Field(..., alias="initialTrigger", description="Trigger to supply to all tasks to be made")
    type: constr(strict=True, min_length=1) = Field(..., description="The type of task activity")
    correlation_ids: Optional[conlist(EventHandlerMapping)] = Field(None, alias="correlationIds", description="The event to correlation ID mappings")
    task_fields: Optional[Dict[str, FieldMapping]] = Field(None, alias="taskFields", description="The event to task field mappings")
    __properties = ["initialTrigger", "type", "correlationIds", "taskFields"]

    @validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('CreateNewTask', 'UpdateMatchingTasks'):
            raise ValueError("must be one of enum values ('CreateNewTask', 'UpdateMatchingTasks')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateNewTaskActivity:
        """Create an instance of CreateNewTaskActivity from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in correlation_ids (list)
        _items = []
        if self.correlation_ids:
            for _item in self.correlation_ids:
                if _item:
                    _items.append(_item.to_dict())
            _dict['correlationIds'] = _items
        # override the default output from pydantic by calling `to_dict()` of each value in task_fields (dict)
        _field_dict = {}
        if self.task_fields:
            for _key in self.task_fields:
                if self.task_fields[_key]:
                    _field_dict[_key] = self.task_fields[_key].to_dict()
            _dict['taskFields'] = _field_dict
        # set to None if correlation_ids (nullable) is None
        # and __fields_set__ contains the field
        if self.correlation_ids is None and "correlation_ids" in self.__fields_set__:
            _dict['correlationIds'] = None

        # set to None if task_fields (nullable) is None
        # and __fields_set__ contains the field
        if self.task_fields is None and "task_fields" in self.__fields_set__:
            _dict['taskFields'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateNewTaskActivity:
        """Create an instance of CreateNewTaskActivity from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateNewTaskActivity.parse_obj(obj)

        _obj = CreateNewTaskActivity.parse_obj({
            "initial_trigger": obj.get("initialTrigger"),
            "type": obj.get("type"),
            "correlation_ids": [EventHandlerMapping.from_dict(_item) for _item in obj.get("correlationIds")] if obj.get("correlationIds") is not None else None,
            "task_fields": dict(
                (_k, FieldMapping.from_dict(_v))
                for _k, _v in obj.get("taskFields").items()
            )
            if obj.get("taskFields") is not None
            else None
        })
        return _obj



from lusid_workflow.extensions.api_client_factory import SyncApiClientFactory, ApiClientFactory
from lusid_workflow.extensions.configuration_loaders import (
    ConfigurationLoader,
    SecretsFileConfigurationLoader,
    EnvironmentVariablesConfigurationLoader,
    ArgsConfigurationLoader,
)
from lusid_workflow.extensions.api_client import SyncApiClient

__all__ = [
    "SyncApiClientFactory",
    "ApiClientFactory",
    "ConfigurationLoader",
    "SecretsFileConfigurationLoader",
    "EnvironmentVariablesConfigurationLoader",
    "ArgsConfigurationLoader",
    "SyncApiClient"
]


import aiohttp
from typing import List, Tuple, Any, Union
import socket
from urllib3 import HTTPSConnectionPool, HTTPConnectionPool
from lusid_workflow.extensions.socket_keep_alive import TCP_KEEPALIVE_INTERVAL, TCP_KEEP_IDLE


def adjust_connection_socket(conn):
    """
    Adjusts the socket settings so that the client sends a TCP keep alive probe over the connection. This is only
    applied where possible, if the ability to set the socket options is not available, for example using Anaconda,
    then the settings will be left as is.

    :param conn: The connection to update the socket settings for
    :param str protocol: The protocol of the connection

    :return: None
    """
    try:
        # set TCP keep alive interval on windows connections
        conn.ioctl(
            socket.SIO_KEEPALIVE_VALS,
            (1, TCP_KEEP_IDLE * 1000, TCP_KEEPALIVE_INTERVAL * 1000),
        )
    except AttributeError:
        pass


class TcpKeepAliveConnector(aiohttp.TCPConnector):
    """Replacement for aiohttp.TCPConnector
    which sets socket options on each connection.
    So we can use tcp keep alives which aiohttp has limited support for.
    """
    def __init__(
        self,
        connector: aiohttp.TCPConnector,
        socket_options: Union[Tuple[Any, Any, Any], Tuple[Any, Any, None, int]]
    ) -> None:
        self.__connector = connector
        self.socket_options = socket_options or []

    @property
    def _timeout_ceil_threshold(self):
        return self.__connector._timeout_ceil_threshold

    @property
    def _loop(self):
        return self.__connector._loop

    @property
    def closed(self):
        return self.__connector.closed

    async def close(self) -> None:
        await self.__connector.close()

    async def connect(
        self,
        req: aiohttp.ClientRequest,
        traces: List[aiohttp.tracing.Trace],
        timeout: aiohttp.ClientTimeout,
    ) -> aiohttp.connector.Connection:
        """Wraps TCP connector, each new connection will have socket options
        and windows ioctl for keep alives applied

        Parameters
        ----------
        req : aiohttp.ClientRequest
        traces : List[aiohttp.tracing.Trace]
        timeout : aiohttp.ClientTimeout
        """
        connection = await self.__connector.connect(req, traces, timeout)
        sock = connection.protocol.transport.get_extra_info("socket")
        for option in self.socket_options:
            sock.setsockopt(*option)
        adjust_connection_socket(sock)
        return connection


class TCPKeepAliveHTTPSConnectionPool(HTTPSConnectionPool):
    """
    This class overrides the _validate_conn method in the HTTPSConnectionPool class. This is the entry point to use
    for modifying the socket as it is called after the socket is created and before the request is made.
    """

    def _validate_conn(self, conn):
        """
        Called right before a request is made, after the socket is created.
        """
        # Call the method on the base class
        super()._validate_conn(conn)

        # Set up TCP Keep Alive probes, this is the only line added to this function
        adjust_connection_socket(conn)


class TCPKeepAliveHTTPConnectionPool(HTTPConnectionPool):
    """
    This class overrides the _validate_conn method in the HTTPSConnectionPool class. This is the entry point to use
    for modifying the socket as it is called after the socket is created and before the request is made.

    In the base class this method is passed completely.
    """

    def _validate_conn(self, conn):
        """
        Called right before a request is made, after the socket is created.
        """
        # Call the method on the base class
        super()._validate_conn(conn)

        # Set up TCP Keep Alive probes, this is the only line added to this function
        adjust_connection_socket(conn)
